﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Web;
using System.Net;
using XServer;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        private System.Drawing.Point lastTooltip = new System.Drawing.Point(-1, -1);
        XMapWSService svcMap = new XMapWSService()
        {
            Credentials = new NetworkCredential(Properties.Settings.Default.xMap_Username,Properties.Settings.Default.xMap_Password),
            Url = Properties.Settings.Default.xMap_Urls
        };

        Layer[] arrLayer;

        Dictionary<string, PlainPoint> dictCities = new Dictionary<string, PlainPoint>();

        System.DateTime referenceTime = Properties.Settings.Default.MapParams_ReferenceTime;

        MapSection mapSection = new MapSection()
        {
            center = new XServer.Point()
            {
                wkt = Properties.Settings.Default.MapSection_Center_WKT,                
            },
            scale = Properties.Settings.Default.MapSection_Scale
        };
        ImageInfo ii = new ImageInfo()
        {
            format = ImageFileFormat.JPG
        };

        Map map = null;
        int TooltipSize = Properties.Settings.Default.TooltipSize;
        List<LayerObject> lstLayerObjects;

        CallerContext cc = new CallerContext();
        
        
        public int SCROLL = Properties.Settings.Default.SCROLL;

        public Form1()
        {
            InitializeComponent();
            List<FeatureLayer> lstLayer = new List<FeatureLayer>();
            foreach (string layername in Properties.Settings.Default.LAYER.Split(','))
            {
                ToolStripMenuItem tsmi = new ToolStripMenuItem(layername, null, this.layersToolStripMenuItem_Click);
                tsmi.Checked = false;
                layersToolStripMenuItem.DropDownItems.Add(tsmi);
                lstLayer.Add(new FeatureLayer()
                {
                    visible = tsmi.Checked,
                    objectInfos = ObjectInfoType.REFERENCEPOINT,
                    name = layername
                });
            }
            arrLayer = lstLayer.ToArray();
            //
            fillDummyCities();
            //
            fillDummyUrls();
            //
            foreach (string profile in Properties.Settings.Default.Profiles.Split(','))
            {
                ToolStripMenuItem tsmiProfile = new ToolStripMenuItem(profile, null, this.profileToolStripMenuItem_Click);
                tsmiProfile.Checked = false;
                profileToolStripMenuItem.DropDownItems.Add(tsmiProfile);
            }
            //
            cc.wrappedProperties = new CallerContextProperty[] {
                new CallerContextProperty(){
                    key = "ProfileXMLSnippet",
                    value = Properties.Settings.Default.ProfileXMLSnippet
                },
                new CallerContextProperty(){
                    key = "Profile",
                    value = "Default"
                }
            };


            render();
        }

        public void fillDummyUrls()
        {
            foreach (string ownUrl in Properties.Settings.Default.xMap_Urls.Split(';'))
            {
                ToolStripMenuItem tsmiService = new ToolStripMenuItem(ownUrl, null, servicesToolStripMenuItem_Click);
                servicesToolStripMenuItem.DropDownItems.Add(tsmiService);
            }
        }

        public void fillDummyCities()
        {
            dictCities.Add("Amsterdam (NDL)", new PlainPoint() { x = 544550.0, y = 6860351.0 });
            dictCities.Add("Berlin (GER)", new PlainPoint() { x = 1486836.0, y = 6886322.0 });
            dictCities.Add("Cannberra (AUS)", new PlainPoint() { x = 16592230.0, y = -4196081.0 });
            dictCities.Add("London (GBR)", new PlainPoint() { x = -14233.0, y = 6703968.0 });
            dictCities.Add("Madrid (ESP)", new PlainPoint() { x = -411200.0, y = 4922105.0 });
            dictCities.Add("Moscow (RUS)", new PlainPoint() { x = 4182482.0, y = 7501968.0 });
            dictCities.Add("Munich (GER)", new PlainPoint() { x = 1287336.0, y = 6122663.0 });
            dictCities.Add("Warashaw (POL)", new PlainPoint() { x = 2335978.0, y = 6834283.0 });
            dictCities.Add("Washington (USA)", new PlainPoint() { x = -8566022.0, y = 4700866.0 });

            foreach (string city in dictCities.Keys)
            { 
                ToolStripMenuItem tsmiSampleCity = new ToolStripMenuItem(city,null,sampleCityToolStripMenuItem_Click);
                sampleCitiesToolStripMenuItem.DropDownItems.Add(tsmiSampleCity);
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
//            render();            
        }

        public void render()
        {
            try
            {
                ii.height = pictureBox1.Height;
                ii.width = pictureBox1.Width;
                MapParams mapParams = new MapParams() { 
                    referenceTime = referenceTime.ToString("o")
                };
                referenceTimeToolStripMenuItem.Text = referenceTime.ToString("o");
                DateTime dtStart = DateTime.Now;
                map = svcMap.renderMap(mapSection, mapParams, ii, (from layer in arrLayer where layer.visible select layer).ToArray()                    
                    , true, cc);
                DateTime dtStop = DateTime.Now;
                this.Text = dtStop.Subtract(dtStart).TotalMilliseconds.ToString() + "ms";
                pictureBox1.Image = System.Drawing.Image.FromStream(new System.IO.MemoryStream(map.image.rawImage));
                mapSection = new MapSection()
                {
                    center = map.visibleSection.center,
                    scale = map.visibleSection.scale,
                    scrollHorizontal = 0,
                    scrollVertical = 0,
                    zoom = 0
                };
                lstLayerObjects = new List<LayerObject>();
                if (map.wrappedObjects != null)
                {
                    Graphics gr = System.Drawing.Graphics.FromImage(pictureBox1.Image);
                    //if (Properties.Settings.Default.TooltipShow)
                    //{
                    //    
                    //    for (int x = 0; x <= ii.width; x += 100)
                    //    {
                    //        for (int y = 0; y <= ii.height; y += 100)
                    //        {
                    //            gr.FillRectangle(Brushes.Red, new Rectangle(x - TooltipSize, y - TooltipSize, 2 * TooltipSize, 2 * TooltipSize));
                    //        }
                    //    }
                    //}

                    statusStrip1.Items.Clear();
                    int tooltipCount = 0;
                    foreach (ObjectInfos curOI in map.wrappedObjects)
                    {
                        tooltipCount += curOI.wrappedObjects.Length;
                    }
                    if (tooltipCount <= Properties.Settings.Default.TooltipMaxcount)
                    {
                        for (int indexLayer = 0; indexLayer < map.wrappedObjects.Length; indexLayer++)
                        {
                            ObjectInfos cOI = map.wrappedObjects[indexLayer];

                            statusStrip1.Items.Add(new ToolStripMenuItem(cOI.name + "(" + cOI.wrappedObjects.Length + ")"));
                            for (int indexObject = 0; indexObject < cOI.wrappedObjects.Length; indexObject++)
                            {
                                LayerObject cLO = cOI.wrappedObjects[indexObject];
                                lstLayerObjects.Add(new LayerObject()
                                {
                                    descr = cOI.name + ":" + cLO.descr,// + "(" + cLO.pixel.x + "/" + cLO.pixel.y + ")",
                                    geometry = cLO.geometry,
                                    hiId = cLO.hiId,
                                    loId = cLO.loId,
                                    pixel = new PlainPoint() { x = cLO.pixel.x, y = cLO.pixel.y },
                                    @ref = cLO.@ref
                                });
                                if (Properties.Settings.Default.TooltipShow)
                                    gr.DrawRectangle(SystemPens.ActiveBorder, new Rectangle((int)cLO.pixel.x - TooltipSize, (int)cLO.pixel.y - TooltipSize, 2 * TooltipSize, 2 * TooltipSize));
                            }
                        }
                    }
                    else
                    {
                        statusStrip1.Items.Add("number of objects in visible section = " + tooltipCount + " (maximum=" + Properties.Settings.Default.TooltipMaxcount +")");
                    }
                }
            }
            catch (System.Web.Services.Protocols.SoapException ex)
            {
                MessageBox.Show(ex.Message, "System.Web.Services.Protocols.SoapException");
            }
            catch (System.Net.WebException ex)
            {
                string message = "System.Net.WebException: " + ex.Message;
                if (ex.InnerException != null)
                    message += "\r\n\r\n" + ex.InnerException.Message;
                MessageBox.Show(message);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Exception");
            }
        }

        public int distance2(System.Drawing.Point p1,PlainPoint p2)
        {
            return Math.Max(Math.Abs(p1.X - (int)p2.x) , Math.Abs(p1.Y - (int)p2.y));
            int x1 = p1.X, y1 = p1.Y, x2 = (int)p2.x, y2 = (int)p2.y;
            return (x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2);
        }

        
        private void Form1_KeyPress(object sender, KeyPressEventArgs e)
        {
            bool toRender = true;
            if (e.KeyChar == '+')
            {
                mapSection.scale /= 2;
                if (mapSection.scale < 5)
                    mapSection.scale = 5; 
            }
            else if (e.KeyChar == '-')
                mapSection.scale *= 2;
            // move
            else if (e.KeyChar == '2')
                mapSection.scrollVertical = -SCROLL;
            else if (e.KeyChar == '4')
                mapSection.scrollHorizontal = -SCROLL;
            else if (e.KeyChar == '6')
                mapSection.scrollHorizontal = SCROLL;
            else if (e.KeyChar == '8')
                mapSection.scrollVertical = SCROLL;
            else if (e.KeyChar == Properties.Settings.Default.ReferenceTime_ADD_HOUR)
                referenceTime = referenceTime.AddHours(1.0);
            else if (e.KeyChar == Properties.Settings.Default.ReferenceTime_SUBTRACT_HOUR)
                referenceTime = referenceTime.AddHours(-1.0);
            else if (e.KeyChar == Properties.Settings.Default.ReferendeTime_SUBTRACT_DAY)
                referenceTime = referenceTime.AddDays(-1.0);
            else if (e.KeyChar == Properties.Settings.Default.ReferenceTime_ADD_DAY)
                referenceTime = referenceTime.AddDays(1.0);
            else
                toRender = false;
            if (toRender)
                render();

        }

        private void pictureBox1_Click(object sender, EventArgs e)
        {
            MouseEventArgs mea = (MouseEventArgs)e;
            double x = Convert.ToDouble(mea.X),y = Convert.ToDouble(mea.Y);
            double width = (double)pictureBox1.Width , height = (double)pictureBox1.Height;
            double scrollx = -100.0 + 200.0 * (x / width), scrolly = 100.0 - 200.0 * (y / height);
            mapSection.scrollHorizontal = (int)scrollx;
            mapSection.scrollVertical = (int)scrolly;
            render();
        }

        private void Form1_ResizeEnd(object sender, EventArgs e)
        {
            //render();
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            render();
        }

        private void Form1_ResizeBegin(object sender, EventArgs e)
        {
            render();
        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs mea)
        {
            this.Text = mea.X.ToString() + " / " + mea.Y;
            if (mea.Location != lastTooltip)
            {
                lastTooltip = mea.Location;
                List<string> lstLabels = new List<string>();//(from labels in lstLayerObjects where distance(mea.Location, labels.pixel) < 20 select labels.descr).ToArray());
                if (lstLayerObjects != null)
                {
                    foreach (LayerObject cLO in lstLayerObjects)
                    {
                        if (distance2(mea.Location, cLO.pixel) <= TooltipSize)
                            lstLabels.Add(cLO.descr);
                    }


                    string label = String.Join("\r\n", lstLabels.ToArray());
                    if (lstLabels.Count > 0)
                    {
                        toolTip1.Active = true;
                        toolTip1.ShowAlways = true;
                        toolTip1.ToolTipIcon = ToolTipIcon.Info;
                        toolTip1.ToolTipTitle = "ObjectInfos";
                        toolTip1.Show(label, pictureBox1, mea.Location, 10000);
                        //toolTip1.SetToolTip(pictureBox1, label);

                        //MessageBox.Show();
                        pictureBox1.Update();
                    }
                    else
                        toolTip1.Hide(this);
                }
            }
        }

        private void layersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < arrLayer.Length; i++)
            {
                ToolStripMenuItem tsmi = ((ToolStripMenuItem)sender);
                if (arrLayer[i].name == tsmi.Text)
                {
                    bool newStatus = !arrLayer[i].visible;
                    arrLayer[i].visible = newStatus;
                    tsmi.Checked = newStatus;
                    render();
                }
            }
        }
        private void sampleCityToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem tsmi = ((ToolStripMenuItem)sender);
            mapSection.center.point = dictCities[tsmi.Text];
            render();
        }
        private void servicesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem tsmi = ((ToolStripMenuItem)sender);
            svcMap.Url = tsmi.Text;
            render();
        }
        private void profileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem tsmi = ((ToolStripMenuItem)sender);
            cc.wrappedProperties[1].value = tsmi.Text;
            render();
        }
    }
}
